==> express-di-0.0.7.tgz: package/lib/di.js <==
var express   = require('express');
var utils     = require('./utils');
var mapSeries = require('async').mapSeries;

var factories = {};
express.application.factory = function(name, fn) {
  if (typeof fn !== 'function') {
    throw new Error('app.factory() requires a function but got a ' + typeof fn);
  }
  factories[name] = fn;
};

var route = express.Router.prototype.route;

express.Router.prototype.route = function(method, path) {
  var callbacks = utils.flatten([].slice.call(arguments, 2));

  callbacks = callbacks
    .filter(function(fn) { return typeof fn === 'function'; })
    .map(function(fn) {
      var parameters = utils.getParameters(fn);

      if (!utils.needInject(parameters)) {
        return fn;
      }

      return function(req, res, next) {
        if (!req.__di_caches) {
          req.__di_caches = {};
        }
        var self = this;
        mapSeries(parameters, function(dependency, callback) {
          if (dependency === 'req')  return callback(null, req);
          if (dependency === 'res')  return callback(null, res);
          if (dependency === 'next') return callback(null, next);
          factory = factories[dependency];
          if (!factory) {
            throw new Error('Unrecognized dependency: ' + dependency);
          }
          if (req.__di_caches[dependency]) {
            callback(req.__di_caches[dependency][0], req.__di_caches[dependency][1]);
          } else {
            factory(req, res, function(err, result) {
              req.__di_caches[dependency] = [err, result];
              callback(err, result);
            });
          }
        }, function(err, results) {
          if (err) {
            return next(err);
          }
          fn.apply(self, results);
        });
      };
    });

  route.call(this, method, path, callbacks);
};

exports.utils = utils;

==> express-di-0.0.9.tgz: package/lib/di.js <==
var express   = require('express');
var utils     = require('./utils');
var mapSeries = require('async').mapSeries;

var diFactoriesKey = '__EXPRESS_DI_FACTORIES__';
express.application.factory = function(name, fn) {
  if (typeof fn !== 'function') {
    throw new Error('app.factory() requires a function but got a ' + typeof fn);
  }
  var factories = this.get(diFactoriesKey);
  if (!factories) {
    factories = {};
    this.set(diFactoriesKey, factories);
  }
  factories[name] = fn;
};

var route = express.Router.prototype.route;

express.Router.prototype.route = function(method, path) {
  var callbacks = utils.flatten([].slice.call(arguments, 2));

  callbacks = callbacks
    .map(function(fn) {
      if (typeof fn !== 'function') return fn;

      var parameters = utils.getParameters(fn);

      if (!utils.needInject(parameters)) {
        return fn;
      }

      return function(req, res, next) {
        if (!req.__di_caches) {
          req.__di_caches = {};
        }
        var self = this;
        mapSeries(parameters, function(dependency, callback) {
          if (dependency === 'req')  return callback(null, req);
          if (dependency === 'res')  return callback(null, res);
          if (dependency === 'next') return callback(null, next);
          var factories = req.app.get(diFactoriesKey);
          factory = factories && factories[dependency];
          if (!factory) {
            throw new Error('Unrecognized dependency: ' + dependency);
          }
          if (req.__di_caches[dependency]) {
            callback(req.__di_caches[dependency][0], req.__di_caches[dependency][1]);
          } else {
            factory(req, res, function(err, result) {
              req.__di_caches[dependency] = [err, result];
              callback(err, result);
            });
          }
        }, function(err, results) {
          if (err) {
            return next(err);
          }
          fn.apply(self, results);
        });
      };
    });

  route.call(this, method, path, callbacks);
};

exports.utils = utils;
